#include "SceneOpenGL.h"

SceneOpenGL::SceneOpenGL(std::string titre, int largeur, int hauteur)
: m_titreFenetre(titre), m_largeurFenetre(largeur), m_hauteurFenetre(hauteur), m_fenetre(0), m_contexteOpenGL(0)
{

}

SceneOpenGL::~SceneOpenGL()
{
    SDL_GL_DeleteContext(m_contexteOpenGL);
    SDL_DestroyWindow(m_fenetre);
    SDL_Quit();
}

bool SceneOpenGL::initialiserFenetre()
{
    // Initialisation de la SDL
    if(SDL_Init(SDL_INIT_VIDEO) < 0)
    {
        std::cout << "Erreur lors de l'initialisation de la SDL : " << SDL_GetError() << std::endl;
        SDL_Quit();

        return false;
    }


    // Version d'OpenGL
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
    SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);


    // Double Buffer
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);


    // Création de la fenêtre
    m_fenetre = SDL_CreateWindow(m_titreFenetre.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                                 m_largeurFenetre, m_hauteurFenetre, SDL_WINDOW_SHOWN | SDL_WINDOW_OPENGL);
    if(m_fenetre == 0)
    {
        std::cout << "Erreur lors de la creation de la fenetre : " << SDL_GetError() << std::endl;
        SDL_Quit();

        return false;
    }


    // Création du contexte OpenGL
    m_contexteOpenGL = SDL_GL_CreateContext(m_fenetre);
    if(m_contexteOpenGL == 0)
    {
        std::cout << SDL_GetError() << std::endl;
        SDL_DestroyWindow(m_fenetre);
        SDL_Quit();

        return false;
    }

    return true;
}

bool SceneOpenGL::initGL()
{
    //glEnable(GL_DEPTH_TEST);
    return true;
}

void SceneOpenGL::bouclePrincipale()
{
    // Shader
    Shader shader("Shaders/basique2D.vert", "Shaders/basique2D.frag");
    shader.charger();

    // Vertices
    float vertices[] = {-0.5, -0.5,   0.5, -0.5,   0.5, 0.5,    // Triangle 1
                        -0.5, -0.5,   -0.5, 0.5,   0.5, 0.5};   // Triangle 2

    /* ***** Gestion du VBO ***** */
    GLuint vbo;
    int tailleVerticesBytes = 12 * sizeof(float);

    // Génération du VBO
    glGenBuffers(1, &vbo);

    // Verrouillage
    glBindBuffer(GL_ARRAY_BUFFER, vbo);

        // Remplissage
        glBufferData(GL_ARRAY_BUFFER, tailleVerticesBytes, 0, GL_STATIC_DRAW);
        glBufferSubData(GL_ARRAY_BUFFER, 0, tailleVerticesBytes, vertices);

    // Déverrouillage
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    /* ***** Gestion du VAO ***** */
    GLuint vao;

    // Génération du VAO
    glGenVertexArrays(1, &vao);

    // Verrouillage du VAO
    glBindVertexArray(vao);

        // Verrouillage du VBO
        glBindBuffer(GL_ARRAY_BUFFER, vbo);

            // Vertex Attrib 0 (Vertices)
            glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0));
            glEnableVertexAttribArray(0);

        // Déverrouillage du VBO
        glBindBuffer(GL_ARRAY_BUFFER, 0);

    // Déverrouillage du VAO
    glBindVertexArray(0);

    // Variables relatives au framerate
    unsigned int frameRate (1000 / 50);
    Uint32 debutBoucle(0), finBoucle(0), tempsEcoule(0);

    // Boucle principale
    while(!m_input.terminer())
    {
        // On définit le temps de début de boucle
        debutBoucle = SDL_GetTicks();

        // Gestion des évènements
        m_input.updateEvenements();
        if(m_input.getTouche(SDL_SCANCODE_ESCAPE))
           break;

        // Nettoyage de l'écran
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        // Shader verrouillé
        glUseProgram(shader.getProgramID());

            // Verrouillage du VAO
            glBindVertexArray(vao);

            // Rendu
            glDrawArrays(GL_TRIANGLES, 0, 6);

            // Déverrouillage du VAO
            glBindVertexArray(0);

        // Shader dévérouillé
        glUseProgram(0);

        // Actualisation de la fenêtre
        SDL_GL_SwapWindow(m_fenetre);

        // Calcul du temps écoulé
        finBoucle = SDL_GetTicks();
        tempsEcoule = finBoucle - debutBoucle;

        // Si nécessaire, on met en pause le programme
        if(tempsEcoule < frameRate)
            SDL_Delay(frameRate - tempsEcoule);
    }

    // Destruction du VAO et du VBO
    glDeleteBuffers(1, &vbo);
    glDeleteVertexArrays(1, &vao);
}

